import { processMaybeMutable } from "@mutablejs/core";
function propProcessors(element) {
  return {
    value(data) {
      if (element instanceof HTMLInputElement || element instanceof HTMLTextAreaElement || element instanceof HTMLSelectElement) {
        element.value = data;
      } else {
        element.setAttribute("value", data);
      }
    },
    style(data) {
      if (typeof data === "string") {
        element.setAttribute("style", data);
      } else {
        Object.entries(data).forEach(([key, val]) => {
          processMaybeMutable((val2) => {
            element.style[key] = `${val2}` || "";
          })(val);
        });
      }
    }
  };
}
function isChild(test) {
  return !!(typeof test === "string" && test) || typeof test === "number" || test instanceof Node;
}
function mutableElement(tag, props, ...children) {
  const element = document.createElement(tag);
  const processors = propProcessors(element);
  props && Object.entries(props).forEach(([propName, propValue]) => {
    if (element.hasAttribute(propName)) {
      return;
    }
    if (processors.hasOwnProperty(propName)) {
      if (propName === "value") {
        processMaybeMutable(processors.value)(propValue);
      } else {
        processMaybeMutable(processors[propName])(propValue);
      }
    } else if (propName.startsWith("on")) {
      processMaybeMutable((action) => {
        element[propName] = action;
      })(propValue);
    } else {
      processMaybeMutable((value) => {
        if (!value && value !== "" && value !== 0) {
          element.removeAttribute(propName);
        } else {
          element.setAttribute(propName, value);
        }
      })(propValue);
    }
  });
  const childOffset = [];
  children && children.forEach((childData, index) => {
    processMaybeMutable((nodeData) => {
      let nodes = [];
      let currentOffset = 0;
      for (let i = 0; i < index; i++) {
        currentOffset += childOffset[i];
      }
      if (Array.isArray(nodeData)) {
        nodes = nodeData.filter(isChild);
      } else if (isChild(nodeData)) {
        nodes = [nodeData];
      }
      for (let i = 0; i < childOffset[index]; i++) {
        const index2 = currentOffset;
        const child = element.childNodes[index2];
        element.removeChild(child);
      }
      const insertBefore = element.childNodes[currentOffset];
      childOffset[index] = nodes.length;
      nodes.forEach((child) => {
        const node = typeof child === "string" || typeof child === "number" ? document.createTextNode(`${child}`) : child;
        element.insertBefore(node, insertBefore);
      });
    })(childData);
  });
  return element;
}
export { mutableElement as default, mutableElement };
