(function(global, factory) {
  typeof exports === "object" && typeof module !== "undefined" ? factory(exports, require("@mutablejs/core")) : typeof define === "function" && define.amd ? define(["exports", "@mutablejs/core"], factory) : (global = typeof globalThis !== "undefined" ? globalThis : global || self, factory(global["@mutablejs/dom"] = {}, global.core));
})(this, function(exports2, core) {
  "use strict";
  function propProcessors(element) {
    return {
      value(data) {
        if (element instanceof HTMLInputElement || element instanceof HTMLTextAreaElement || element instanceof HTMLSelectElement) {
          element.value = data;
        } else {
          element.setAttribute("value", data);
        }
      },
      style(data) {
        if (typeof data === "string") {
          element.setAttribute("style", data);
        } else {
          Object.entries(data).forEach(([key, val]) => {
            core.processMaybeMutable((val2) => {
              element.style[key] = `${val2}` || "";
            })(val);
          });
        }
      }
    };
  }
  function isChild(test) {
    return !!(typeof test === "string" && test) || typeof test === "number" || test instanceof Node;
  }
  function mutableElement(tag, props, ...children) {
    const element = document.createElement(tag);
    const processors = propProcessors(element);
    props && Object.entries(props).forEach(([propName, propValue]) => {
      if (element.hasAttribute(propName)) {
        return;
      }
      if (processors.hasOwnProperty(propName)) {
        if (propName === "value") {
          core.processMaybeMutable(processors.value)(propValue);
        } else {
          core.processMaybeMutable(processors[propName])(propValue);
        }
      } else if (propName.startsWith("on")) {
        core.processMaybeMutable((action) => {
          element[propName] = action;
        })(propValue);
      } else {
        core.processMaybeMutable((value) => {
          if (!value && value !== "" && value !== 0) {
            element.removeAttribute(propName);
          } else {
            element.setAttribute(propName, value);
          }
        })(propValue);
      }
    });
    const childOffset = [];
    children && children.forEach((childData, index) => {
      core.processMaybeMutable((nodeData) => {
        let nodes = [];
        let currentOffset = 0;
        for (let i = 0; i < index; i++) {
          currentOffset += childOffset[i];
        }
        if (Array.isArray(nodeData)) {
          nodes = nodeData.filter(isChild);
        } else if (isChild(nodeData)) {
          nodes = [nodeData];
        }
        for (let i = 0; i < childOffset[index]; i++) {
          const index2 = currentOffset;
          const child = element.childNodes[index2];
          element.removeChild(child);
        }
        const insertBefore = element.childNodes[currentOffset];
        childOffset[index] = nodes.length;
        nodes.forEach((child) => {
          const node = typeof child === "string" || typeof child === "number" ? document.createTextNode(`${child}`) : child;
          element.insertBefore(node, insertBefore);
        });
      })(childData);
    });
    return element;
  }
  exports2["default"] = mutableElement;
  exports2.mutableElement = mutableElement;
  Object.defineProperties(exports2, { __esModule: { value: true }, [Symbol.toStringTag]: { value: "Module" } });
});
